#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/resource.h>

#include <linux/i2c.h>
#include <linux/i2c-dev.h>

static int zone = 1;

int verbose;

static void
serial_set_mark (int mark)
{
  char buf[1];
  buf[0] = mark;
  i2c(zone, 5, 1, buf, 1);
}

static void
serial_write (char *buf, int len)
{
  i2c(zone, 6, 1, buf, len);
}

static unsigned char last_status = 0;

static int
get_status ()
{
  unsigned char msgbuf[1];
  i2c(zone, 2, 0, msgbuf, 1);
  if (msgbuf[0] != last_status)
    {
      printf("[%02x]", msgbuf[0]);
      last_status = msgbuf[0];
    }
  return msgbuf[0];
}

#define buffer_room()  (get_status() & 0x40)
#define buffer_empty() (get_status() & 0x20)

int
main(int argc, char **argv)
{
  int i, wi=0, ri=0;
  int fill_time, mark_time, empty_time;
  int buf_size = 0;

  //setbuf(stdout, 0);
  i2c_open ();

  zone = 1;

  /*  i = get_status();
  printf("status %02x room %d empty %d\n",
  i, i & 0x40, i & 0x20); */

  serial_set_mark (4);
  msclock();
  printf("filling...\n");
  do
    {
      serial_write ("123456789abcdefg", 16);
      buf_size += 16;
    } while (buffer_room ());
  fill_time = msclock();
  printf("waiting for room...\n");
  while (!buffer_room())
    usleep (100);
  mark_time = msclock();
  printf("waiting for empty...\n");
  while (!buffer_empty())
    usleep (100);
  empty_time = msclock();

  printf("buffer %d fill %d mark %d empty %d\n",
	 buf_size, fill_time, mark_time, empty_time);

  i2c_close ();
  return 0;
}
