#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <termios.h>

#include "serial.h"

static char *port;
static int fd;
static int timeout = 1000;
int verbose;

static int b16 = 0;

void
serial_init(char *_port, int baud)
{
  struct termios attr;

  switch (baud)
    {
    case 9600:
    case 96:
      baud = B9600;
      break;
    case 19200:
    case 192:
    case 19:
      baud = B19200;
      break;
    case 0:
    case 38400:
    case 384:
    case 38:
      baud = B38400;
      break;
    case 57600:
    case 576:
    case 57:
      baud = B57600;
      break;
    case 115200:
    case 1152:
    case 115:
      baud = B115200;
      break;
    default:
      printf("Unsupported baud rate %d\n", baud);
      exit(1);
    }

  port = _port;
  fd = open(port, O_RDWR|O_NOCTTY|O_NDELAY);
  if (fd < 0)
    {
      printf("can't open %s\n", port);
      perror("The error was");
      exit(1);
    }

  fcntl(fd, F_SETFL, 0);

#if 0
  tcgetattr(fd, &attr);
  cfsetispeed(&attr, 0);
  cfsetospeed(&attr, 0);
  tcsetattr(fd, TCSAFLUSH, &attr);
  /* 1 mSec reset pulse */
  usleep(3*1000);
#endif

  tcgetattr(fd, &attr);
  cfsetispeed(&attr, baud);
  cfsetospeed(&attr, baud);
  cfmakeraw(&attr);

  attr.c_iflag &= ~(IXON | IXOFF | IXANY);
  attr.c_iflag |= IGNBRK;

  attr.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

  attr.c_cflag &= ~(PARENB | CSIZE | CRTSCTS);
  attr.c_cflag |= CS8 | CLOCAL | CSTOPB;
  attr.c_cc[VTIME] = 5;

  tcsetattr(fd, TCSANOW, &attr);

#if 0
  /* 1 mSec delay to let board "wake up" after reset */
  usleep(3*1000);
#endif
}

void
serial_set_timeout (int mseconds)
{
  timeout = mseconds;
}

void
serial_sync ()
{
#if 0
  struct timeval s, e, d;
  gettimeofday(&s, 0);
  printf("%d.%06d : ", s.tv_sec, s.tv_usec);
  write_gpio (66, 1);
#endif
  tcdrain(fd);
#if 0
  write_gpio (66, 0);
  gettimeofday(&e, 0);
  printf("%d.%06d ", e.tv_sec, e.tv_usec);

  d.tv_sec = e.tv_sec - s.tv_sec;
  d.tv_usec = e.tv_usec - s.tv_usec;
  if (d.tv_usec < 0) {
    d.tv_usec += 1000000;
    d.tv_sec -= 1;
  }
  printf(" = %d.%06d\n", d.tv_sec, d.tv_usec);
#endif

  b16 = 0;
}

void
serial_write (unsigned char ch)
{
  if (verbose > 1)
    {
      /*      if (isgraph(ch))
	printf("\033[32m%c\033[0m", ch);
	else */
	printf("\033[36m%02x\033[0m", ch);
	    if (++b16 == 16)
	      {
		printf("\n");
		b16 = 0;
	      }
    }
  write(fd, &ch, 1);
}

void
serial_write_block (unsigned char *ch, int len)
{
  while (len--)
    serial_write(*ch++);
  serial_sync ();
}

void
serial_write_string (char *ch)
{
  while (*ch)
    serial_write(*ch++);
  serial_sync ();
}

void
serial_drain ()
{
  int old_timeout = timeout;
  timeout = 1;
  while (serial_read() != -1)
    ;
  timeout = old_timeout;
}

int
serial_read (void)
{
  volatile unsigned char ch;
  if (timeout)
    {
      fd_set rd;
      struct timeval tv;
      tv.tv_sec = timeout / 1000;
      tv.tv_usec = (timeout % 1000) * 1000;
      FD_ZERO(&rd);
      FD_SET(fd, &rd);
      if (select(fd+1, &rd, 0, 0, &tv) == 0)
	{
	  if (verbose)
	    printf("[TO]");
	  return -1;
	}
    }
  int r = read(fd, (void *)&ch, 1);
  if (r == 1)
    {
       if (verbose > 1)
	{
	  /*	  if (isgraph(ch))
	    printf("\033[31m%c\033[0m", ch);
	    else*/
	    printf("\033[35m%02x\033[0m", ch);
	    if (++b16 == 16)
	      {
		printf("\n");
		b16 = 0;
	      }
	  fflush(stdout);
	}
      return ch;
    }
  if (verbose)
    printf("[EOF]");
  return -1;
}
