#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>

#include "bfd.h"

int verbose = 0;
int watchdog = 0;
unsigned char progmem[65536];
unsigned char progset[256];

int
main(int argc, char **argv)
{
  int i, o;
  bfd *prog;
  FILE *outf;
  asection *s;
  unsigned long mach;
  char *symbase = "elf2map";
  char *is_static = "static ";
  unsigned char checksum = 0;

  outf = stdout;

  while ((o = getopt (argc, argv, "vo:")) != -1)
    switch (o)
      {
      case 'v':
	verbose++;
	break;
      case 'w':
	watchdog++;
	break;
      case 'o':
	outf = fopen(optarg, "w");
	if (!outf)
	  {
	    fprintf(stderr, "unable to open %s for writing\n", optarg);
	    perror("the error was");
	    exit(1);
	  }
	break;
      case '?':
	fprintf (stderr,
                 "usage: run [-v] [-w] [-o output] program"
                 " program\n");
	exit (1);
      }

  memset (progmem, 0xff, sizeof(progmem));

  prog = bfd_openr (argv[optind], 0);
  if (!prog)
    {
      fprintf (stderr, "Can't read %s\n", argv[optind]);
      exit (1);
    }

  if (!bfd_check_format (prog, bfd_object))
    {
      fprintf (stderr, "%s not a m32c program\n", argv[optind]);
      exit (1);
    }
  
  for (s = prog->sections; s; s = s->next)
    {
      char *buf;
      bfd_size_type size;

      if (! (s->flags & SEC_LOAD))
	continue;

      size = bfd_get_section_size (s);
      if (size <= 0)
	continue;

      bfd_vma base = bfd_section_lma (prog, s);
      if (verbose)
	fprintf (stderr, "[load a=%06x s=%06x e=%06x %s]\n",
		 (int) base, (int) size, (int)(base+size-1), bfd_get_section_name (prog, s));
      buf = (char *) malloc (size);
      bfd_get_section_contents (prog, s, buf, 0, size);

      if (strcmp (bfd_get_section_name (prog, s), ".text") == 0)
	{
	  int i;
	  for (i=0; i<size; i++)
	    checksum += (unsigned char) (buf[i]);
	}

      memcpy (progmem + base, buf, size);
      for (i = base/256; i <= (base + size - 1)/256; i++)
	progset[i] = 1;

      free (buf);
    }

  if (!watchdog)
    progmem[0xffff] = 0xff;

  if (verbose)
    for (i=0; i<256; i++)
      if (progset[i])
	{
	  int j;
	  for (j=0; j<256; j++)
	    {
	      if (j % 32 == 0)
		fprintf(stderr, "%06x:", i*256+j);
	      fprintf(stderr, " %02x", progmem[i*256+j]);
	      if (j % 32 == 31)
		fprintf(stderr, "\n");
	    }
	}

  fwrite ("R8C\0", 1, 4, outf);
  fputc (checksum, outf);

  for (i=0; i<256; i++)
    if (progset[i])
      {
	int j;
	fputc(i, outf);
	fwrite(progmem+i*256, 1, 256, outf);
      }

  fclose (outf);

  fprintf(stderr, "%s: ID = %02x %02x %02x %02x %02x %02x %02x,  Watchdog = %02x\n",
	  argv[optind],
	  progmem[0xffdf], progmem[0xffe3], progmem[0xffeb],
	  progmem[0xffef], progmem[0xfff3], progmem[0xfff7],
	  progmem[0xfffb], progmem[0xffff]);

  if (!progset[0xff])
    {
      fprintf(stderr, "Error: %s doesn't have a reset vector\n", argv[optind]);
      return 1;
    }

  return 0;
}
