	;; Most functions are of the form:
	;;
	;; FixType fix_foo(FixType a, FixType b)
	;;
	;; FixType is:
	;;   word foo.i integer part
	;;   word foo.f fraction part
	;; 
	;; The stack looks like this (after enter #0):
	;;  5[fb]	&rv
	;;  7[fb]	a.i
	;;  9[fb]	a.f
	;; 11[fb]	b.i
	;; 13[fb]	b.f
	;;
	;; Functions that take integer arguments get a in r1 and b in
	;; r2.

#define rv_addr	5[fb]
#define a_i	7[fb]
#define a_f	9[fb]
#define b_i	11[fb]
#define b_f	13[fb]
#define rv_i	[a0]
#define rv_f	2[a0]

	.global	_fix_make
_fix_make:
	enter	#0
	mov.w	rv_addr,a0
	mov.w	r1,rv_i
	mov.w	r2,rv_f
	mov.w	a0,r0
	exitd

	.global	_fix_make8
_fix_make8:
	enter	#0
	mov.w	rv_addr,a0
	exts.b	r1l
	mov.w	r1,rv_i
	shl.w	#8,r2
	mov.w	r2,rv_f
	mov.w	a0,r0
	exitd

	.global	_fix_make16
_fix_make16:
	enter	#0
	mov.w	rv_addr,a0
	mov.b	r1h,r0l
	mov.b	r1l,r1h
	mov.b	#0,r1l
	exts.b	r0l
	mov.w	r0,rv_i
	mov.w	r1,rv_f
	mov.w	a0,r0
	exitd

	.global	_fix_neg
_fix_neg:
	enter	#0
	mov.w	rv_addr,a0
	not.w	a_i
	neg.w	a_f
	adcf.w	a_i
	mov.w	a_i, rv_i
	mov.w	a_f, rv_f
	mov.w	a0,r0
	exitd

	.global	_fix_add
_fix_add:
	enter	#0
	mov.w	rv_addr,a0
	mov.w	a_i,r2
	mov.w	a_f,r0
	mov.w	b_i,r3
	mov.w	b_f,r1
	add.w	r1,r0
	adc.w	r3,r2
	mov.w	r2,rv_i
	mov.w	r0,rv_f
	mov.w	a0,r0
	exitd

	.global	_fix_sub
_fix_sub:
	enter	#0
	mov.w	rv_addr,a0
	mov.w	a_i,r2
	mov.w	a_f,r0
	mov.w	b_i,r3
	mov.w	b_f,r1
	sub.w	r1,r0
	sbb.w	r3,r2
	mov.w	r2,rv_i
	mov.w	r0,rv_f
	mov.w	a0,r0
	exitd

;               a_i   a_f
;           x   b_i   b_f
;            ------------
;       signx bfafh bfafl  [1]
;       biafh biafl        [2]
;       bfaih bfail        [3]
; biaih biail              [4]
; -----------------------
;        rv_i  rv_f        [5]

	;; Note that doing a signed multiply is complicated, because
	;; you have to propogate the sign from the MSB to all the
	;; parts of each multiplicand, which is hard.  So, we don't.
	;; We capture the signs separately.

	.global	_fix_mul
_fix_mul:
	enter	#0

	mov.w	a_i,a1
	jpz	a_positive
	not.w	a_i
	neg.w	a_f
	adcf.w	a_i
a_positive:	

	mov.w	b_i,a0
	jpz	b_positive
	not.w	b_i
	neg.w	b_f
	adcf.w	b_i
b_positive:	

	xor.w	a0,a1		; a1 now has the sign of the result

;	jsr.w	console_prints
;	.ascii	"fix_mul\r\n\0"
	mov.w	a_i,r2
	mov.w	a_f,r0
	mov.w	b_i,r3
	mov.w	b_f,r1
;	jsr.w	_dumpregs

	mov.w	b_f,r0
	mulu.w	a_f,r0		; r2r0 holds the bottom 2/3 of [1]
	mov.w	r2,r0
	mov.w	#0,r2
	mov.w	r0,r1
	mov.w	r2,r3		; r3r1 now holds the top 2/3 of [1]
;	jsr.w	_dumpregs

	mov.w	b_i,r0
	mulu.w	a_f,r0		; r2r0 holds [2]
	add.w	r0,r1
	adc.w	r2,r3		; r3r1 holds [1]+[2]+[3]
;	jsr.w	_dumpregs

	mov.w	a_i,r0
	mulu.w	b_f,r0		; r2r0 holds [3]
	add.w	r0,r1
	adc.w	r2,r3		; r3r1 holds [1]+[2]+[3]
;	jsr.w	_dumpregs

	mov.w	a_i,r0
	mulu.w	b_i,r0		; r2r0 holds [4]
	add.w	r0,r3		; r3r1 holds [1]+[2]+[3]+[4]
;	jsr.w	_dumpregs

	btst	15,a1		; check the saved sign bit
	jz	rv_positive
	not.w	r3
	neg.w	r1
	adcf.w	r3
rv_positive:	
;	jsr.w	_dumpregs
	
	mov.w	rv_addr,a0
	mov.w	r3,rv_i
	mov.w	r1,rv_f
	mov.w	a0,r0
	exitd

	.global	_fix_div
_fix_div:
	enter	#2

	mov.w	a_i,a1
	jpz	da_positive
	not.w	a_i
	neg.w	a_f
	adcf.w	a_i
da_positive:	

	mov.w	b_i,a0
	jpz	db_positive
	not.w	b_i
	neg.w	b_f
	adcf.w	b_i
db_positive:	

	xor.w	a0,a1		; a1 now has the sign of the result
	mov.w	a1,-2[fb]	; save it

	;; we want b_i b_f ) 0 a_i a_f 0
	mov.w	#0,r3
	mov.w	a_i,r1
	mov.w	a_f,r2
	mov.w	#0,r0
	mov.w	b_i,a1
	mov.w	b_f,a0
;	jsr.w	console_prints
;	.ascii	"jsr.w to divide64\r\n\0"
;	jsr.w	_dumpregs
	jsr.w	DIVIDE64	; result in r2r0
;	jsr.w	_dumpregs

	mov.w	-2[fb],a1
	jpz	d_rv_pos
	not.w	r2
	neg.w	r0
	adcf.w	r2
d_rv_pos:	
	mov.w	rv_addr,a0
	mov.w	r2,rv_i
	mov.w	r0,rv_f
	mov.w	a0,r0
	exitd

;========================================================================================
;	Title: Dividing 64 bits
;	Outline: Divides 64-bit dividend by 32-bit divisor
;	Input:	------------------------------>	Output:
;	R0 (Lower part of dividend)		R0 (Lower part of quotient)
;	R1 (Upper part of dividend)		R1 (Upper part of quotient)
;	R2 (Middle part of dividend)		R2 (Middle part of quotient)
;	R3 (Most significant part of dividend)	R3 (Most significant part of quotient)
;	A0 (Lower half of divisor)		A0 (Lower half of remainder)
;	A1 (Upper half of divisor)		A1 (Upper half of remainder)
;	Stack amount used: 8 bytes
;	Notes: Division by zero is returned by Z flag.
;	       R3R1R2R0 / A1A0 = R3R1R2R0 remainder A1A0
;========================================================================================
DIVIDE64:
;-------------------------------------------------------
;	Declaration of temporary variables
;-------------------------------------------------------
JYOUYO	=	-6			; Used for remainder calculation
CNT	=	-1			; Shift count counter
	ENTER	#6			; Sets stack frame
	MOV.W	#0,JYOUYO[FB]		; Initializes remainder area
	MOV.W	#0,JYOUYO+2[FB]
	MOV.B	#0,JYOUYO+4[FB]
	CMP.W	#0,A0
	JNE	DIVIDE64_10
	CMP.W	#0,A1
	JEQ	DIVIDE64exit		; --> Division by zero
DIVIDE64_10:
	MOV.B	#64,CNT[FB]		; Sets number of shifts performed (64 times)
DIVIDE64_20:
	SHL.W	#1,R0			; Pushes divided and carry quotient 
	ROLC.W	R2
	ROLC.W	R1
	ROLC.W	R3
	ROLC.W	JYOUYO[FB]		; Creates dividend
	ROLC.W	JYOUYO+2[FB]
	ROLC.B	JYOUYO+4[FB]
	SUB.W	a0,JYOUYO[FB]		; Subtracts divisor
	SBB.W	a1,JYOUYO+2[FB]
	SBB.B	#0,JYOUYO+4[FB]
	BMC	0,R0			; Sets quotient
	JC	DIVIDE64_30		; --> Subtraction of divisor succeeded
	ADD.W	a0,JYOUYO[FB]		; Restored to original data because subtraction of divisor failed
	ADC.W	a1,JYOUYO+2[FB]
	ADCF.B	JYOUYO+4[FB]
DIVIDE64_30:
;	ADJNZ.B	#-1,CNT[FB],DIVIDE64_20	; --> Executes next digit
	add.b	#-1,CNT[FB]
	jnz	DIVIDE64_20
	MOV.W	JYOUYO[FB],a0		; Sets lower half of remainder
	MOV.W	JYOUYO+2[FB],a1		; Sets upper half of remainder
	FCLR	Z			; Division succeeded
DIVIDE64exit:
	EXITD				; Clears stack frame
