.include "r8c-ports.S"

bufsize = 256
bufmask = 0xff

#define TRACE 0
#define ITRACE 0
//#define IPTRACE	mov.w	#__LINE__,iptrace
#define IPTRACE

#define POUND #
#define TC_STATE(x)	mov.w	POUND x,irq_vector+16*4

#define HIGH	or.b #0x10,_p1_bits | mov.b _p1_bits,p1
#define LOW	and.b #0xef,_p1_bits | mov.b _p1_bits,p1
#define MARK	HIGH
#define SPACE	LOW

	.bss
tx_buffer:
	.space	bufsize

	.data
tx_put:	.short	0
tx_get:	.short	0
high:	.short	bufsize/2
	.global _tx_count
_tx_count:	.short	0

	.text
	.global	_serial_init
_serial_init:

;	mov.b	#0x05,u0mr	; UART, 8N1
;	mov.b	#0x00,u0c0	; LSB first, CMOS, buffered, f1
;	mov.b	#0x01,u0c1	; enable transmitter
;	mov.b	#0x30,ucon	; uart0 interrupt on buffer empty.
;	mov.b	#0x81,u0brg	; 9600 baud at f1, 20MHz

	;; Timer C
	mov.w	#2083,tm1	; timer 1 output compare, 9600 baud
	mov.b	#0x58,tcc0	; int3 is int3, no capture, r1, stop
	mov.b	#0xc4,tcc1	; output compare 1 with reload, no filter.
	mov.b	#0x00,tcout	; no output pins.
	mov.b	#7,cmp1ic
	TC_STATE(tc_idle)
;	bset	0,tcc0		; start timer
	SPACE

	mov.w	#6,r1
	mov.w	#i2c_serial_write_cmd,r2
	jsr.w	_i2c_register_command

	mov.w	#5,r1
	mov.w	#i2c_serial_mark_cmd,r2
	jsr.w	_i2c_register_command

#if 0
	mov.w	#irq_uart0tx,irq_vector+0x44
#endif

	rts

#if 0
ping:
	xor.b	#0x80,_p1_bits
	mov.b	_p1_bits,p1
	xor.b	#0x80,_p1_bits
	mov.b	_p1_bits,p1
	rts
#endif

;;; ----------------------------------------------------------------------

i2c_serial_write_cmd:
	.word	0
	.word	serial_write_rx_byte
	.word	0
	.short	0
	.short	0

i2c_serial_mark_cmd:
	.word	0
	.word	serial_mark_rx_byte
	.word	0
	.short	0
	.short	0

serial_write_rx_byte:
	mov.b	#0,r1h
	jmp.w	serial_enqueue

serial_mark_rx_byte:
	mov.b	#0,r1h
	jmp.w	serial_set_space


;;; ----------------------------------------------------------------------

;;; pass the byte in r1l.  ONLY works if there is no other serial traffic,
;;; so that the interrupts never happen.
	.global	_serial_putc
_serial_putc:
	btst	1,u0c1
	jz	_serial_putc
	mov.b	r1l,u0tb
	rts
	.global	serial_putc
serial_putc:
	btst	1,u0c1
	jz	serial_putc
	mov.b	r0l,u0tb
	rts

;;; ----------------------------------------------------------------------

;;; pass the byte in r1l
	.global	serial_enqueue
serial_enqueue:
;	mov.b	#0x71,p1
	pushm	r0,a0
	
	IPTRACE
;	jsr.w	ping
#if TRACE
	mov.b	#'P',r0l
	jsr.w	console_putc
	mov.b	r1l,r0l
	jsr.w	console_putc
#endif
	;; turn on the yellow LED
	and.b	#0xfd,_p1_bits
	mov.b	_p1_bits,p1
	
	mov.w	tx_put,a0
	and.w	#bufmask,a0
	mov.b	r1l,tx_buffer[a0]
	inc.w	a0
	and.w	#bufmask,a0
	mov.w	a0,tx_put

	add.w	#1,_tx_count
	jsr.w	serial_check_space

;	mov.b	_tx_count,r0l
;	jsr.w	console_putc

;	jsr.w	ping
;	jsr.w	irq_uart_sub
;	jsr.w	ping


;	mov.b	#0x72,p1
;	jsr.w	ping
	popm	r0,a0
	rts

;;; pass new low-water margin in r1
	.global	serial_set_space
serial_set_space:
	mov.w	#bufsize, r0
	sub.w	r1,r0
	mov.w	r0,high
;	mov.b	#0xcc, p1
;	mov.b	r0h, p1
	;; fall through

serial_check_space:
	mov.b	_status_byte, r1l
	mov.w	_tx_count,r0
	cmp.w	high,r0
	bmle	6,r1
	cmp.w	#0,r0		; txbuf empty
	bmz	c
;	band	3,u0c0		; and transmitter empty
	band	1,u0c1		; and buffer empty
	bmc	5,r1		; means we're not transmitting
	mov.b	r1l,_status_byte
#if TRACE
	mov.b	#0x0a,r0l
	jsr.w	console_putc
#endif
	rts

;;; ----------------------------------------------------------------------

#if 0

irq_uart_sub:
#if TRACE
	mov.b	#'I',r0l
	jsr.w	console_putc
#endif

;;; STEP 1 - copy as many bytes as we can from the queue to the transmitter.
irq_stuff_bytes:
	cmp.w	#0,_tx_count	; see if the queue is empty
	jz	irq_queue_empty	; it is, done stuffing

	btst	_ti,u0c1	; see if xmit buffer is full
	jz	irq_xmit_full	; it is, done stuffing

	;; stuff a byte into the xmit buffer.
	mov.w	tx_get,a0
#if TRACE
	mov.b	#'G',r0l
	jsr.w	console_putc
	mov.b	tx_buffer[a0],r0l
	jsr.w	console_putc
#endif
	mov.b	tx_buffer[a0],u0tb
	inc.w	a0
	and.w	#bufmask,a0
	mov.w	a0,tx_get
	add.w	#-1,_tx_count

	jmp.b	irq_stuff_bytes

	;; we get here when we've run out of bytes to send, so we want
	;; to wait for the transmitter to finish.
irq_queue_empty:
#if ITRACE
	mov.b	#'q',r0l
	jsr.w	console_putc
#endif
	;; shut off the yellow LED
	or.b	#0x02,_p1_bits
	mov.b	_p1_bits,p1

	btst	_ti,u0c1	; see if transmitter is empty
	jnz	irq_xmit_done	; no, therefore we're done for now

	;; we are still transmitting, wait for it to flush
#if ITRACE
	mov.b	#'f',r0l
	jsr.w	console_putc
#endif
;	bset	0,ucon		; wait for transmitter empty
	mov.b	#5,s0tic	; enable interrupts
	jmp.w	serial_check_space ; done

	;; At this point, we've got nothing further to do.
irq_xmit_done:
#if ITRACE
	mov.b	#'x',r0l
	jsr.w	console_putc
#endif
	mov.b	#0,s0tic	; disable interrupts
	jmp.w	serial_check_space ; done

	;; we get here when the transmitter is full but we have bytes
	;; to send, so we want to wait for the transmit buffer to
	;; empty.
irq_xmit_full:	
#if ITRACE
	mov.b	#'b',r0l
	jsr.w	console_putc
#endif
;	bclr	0,ucon		; wait for buffer empty
	mov.b	#5,s0tic	; enable interrupts
	jmp.w	serial_check_space ; done


;;; ----------------------------------------------------------------------

irq_uart0tx:
	IPTRACE
	pushm	r0,r1,r2,r3,a0,a1
;	jsr.w	do_nothing
#if ITRACE
	mov.b	#'V',r0l
	jsr.w	console_putc
#endif
	jsr.w	irq_uart_sub
;	jsr.w	do_nothing
	nop
	nop
	nop
	nop
	popm	r0,r1,r2,r3,a0,a1
	IPTRACE
	reit

#endif

;;; ----------------------------------------------------------------------

	.global	_run_tx_queue
_run_tx_queue:
	stc	flg,r0
	and.w	#0x8fff,r0
	or.w	#0x5000,r0
	ldc	r0,flg
	bset	0,tcc0		; start timer

1:	btst	0,tcc0
	jnz	1b

	stc	flg,r0
	and.w	#0x8fff,r0
	ldc	r0,flg

	rts

;;; ----------------------------------------------------------------------

	.data
tx_byte:
	.byte	0
	.text

;;; The interrupt vector implements a state machine.  Each state
;;; represents a bit in the bit stream.

tc_idle:
	cmp.w	#0,_tx_count
	jne	1f
	reit

	; we have a byte in the queue, grab it.
1:
	TC_STATE(tc_idle2)
	push.w	a0
	mov.w	tx_get,a0
	mov.b	tx_buffer[a0],tx_byte
	inc.w	a0
	and.w	#bufmask,a0
	mov.w	a0,tx_get
	add.w	#-1,_tx_count
	pop.w	a0
	reit

tc_idle2:
	TC_STATE(tc_bit0)
	MARK
	reit
	
tc_bit0:
	TC_STATE(tc_bit1)
	tst.b	#0x01,tx_byte
	jmp.w	bit_common
	
tc_bit1:
	TC_STATE(tc_bit2)
	tst.b	#0x02,tx_byte
	jmp.w	bit_common
	
tc_bit2:
	TC_STATE(tc_bit3)
	tst.b	#0x04,tx_byte
	jmp.w	bit_common
	
tc_bit3:
	TC_STATE(tc_bit4)
	tst.b	#0x08,tx_byte
	jmp.w	bit_common
	
tc_bit4:
	TC_STATE(tc_bit5)
	tst.b	#0x10,tx_byte
	jmp.w	bit_common
	
tc_bit5:
	TC_STATE(tc_bit6)
	tst.b	#0x20,tx_byte
	jmp.w	bit_common
	
tc_bit6:
	TC_STATE(tc_bit7)
	tst.b	#0x40,tx_byte
	jmp.w	bit_common
	
tc_bit7:
	TC_STATE(tc_space)
	tst.b	#0x80,tx_byte
	jmp.w	bit_common

bit_common:
	jnz	bit_0
	MARK
	reit
bit_0:
	SPACE
	reit

tc_space:
	TC_STATE(tc_idle)
	SPACE
	pushm	r0,r1
	jsr.w	serial_check_space
	popm	r0,r1
	bclr	0,tcc0		; stop timer

	cmp.w	#0,_tx_count
	jne	1f
	;; shut off the yellow LED
	or.b	#0x02,_p1_bits
	mov.b	_p1_bits,p1
1:		
	reit
